<?php

namespace Drupal\image_focus\Plugin\ImageEffect;

use Drupal\Core\Image\ImageInterface;
use Drupal\image\Plugin\ImageEffect\ResizeImageEffect;
use Drupal\image_focus\lib\ImageFocusEntropy;

/**
 * Crops with focus an image resource.
 *
 * @ImageEffect(
 *   id = "image_focus_scale_crop",
 *   label = @Translation("Focus Scale and Crop"),
 *   description = @Translation("Use Image entrophy to detect focal point")
 * )
 */
class FocusScaleCropImageEffect extends ResizeImageEffect {

  /**
   * {@inheritdoc}
   */
  public function applyEffect(ImageInterface $image) {
    $width = $this->configuration['width'];
    $height = $this->configuration['height'];
    [$cx, $cy] = $this->getFocalPoint($image);

    $scale = max($width / $image->getWidth(), $height / $image->getHeight());

    if (!$image->resize($image->getWidth() * $scale, $image->getHeight() * $scale)) {
      $this->logger->error('Image resize failed using the %toolkit toolkit on %path (%mimetype, %dimensions)', [
        '%toolkit' => $image->getToolkitId(),
        '%path' => $image->getSource(),
        '%mimetype' => $image->getMimeType(),
        '%dimensions' => $image->getWidth() . 'x' . $image->getHeight(),
      ]);
      return FALSE;
    }

    $x = max(0, min($image->getWidth() - $width, $cx * $scale - $width / 2));
    $y = max(0, min($image->getHeight() - $height, $cy * $scale - $height / 2));

    if (!$image->crop($x, $y, $width, $height)) {
      $this->logger->error('Image crop failed using the %toolkit toolkit on %path (%mimetype, %dimensions)', [
        '%toolkit' => $image->getToolkitId(),
        '%path' => $image->getSource(),
        '%mimetype' => $image->getMimeType(),
        '%dimensions' => $image->getWidth() . 'x' . $image->getHeight(),
      ]);
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Method getFocalPoint.
   *
   * @param \Drupal\Core\Image\ImageInterface $image
   *   Image.
   *
   * @return array
   *   Return array.
   */
  private function getFocalPoint(ImageInterface $image) {

    $extension = str_replace('jpg', 'jpeg', explode('/', $image->getMimeType())[1]);
    $function = 'imagecreatefrom' . $extension;
    if (function_exists($function) && $resource = $function($image->getSource())) {

      if ($result = $this->getImageEntropy($resource)) {
        return $result;
      }
    }

    return [$image->getWidth() / 2, $image->getHeight() / 2];
  }

  /**
   * Method getImageEntropy.
   *
   * @param mixed $resource
   *   Resource.
   *
   * @return array
   *   Return array.
   */
  private function getImageEntropy($resource) {
    $entropy = new ImageFocusEntropy($resource);
    return $entropy->getFocalPoint();
  }

}
